#include <opencv2/opencv.hpp>
#include <opencv/highgui.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include <fstream>

using namespace std;
using namespace cv;

const bool Show_ROI_bool = false;

//string that contains the exact cascade name

const char* cascade_name = "/usr/local/share/OpenCV/haarcascades/haarcascade_frontalface_alt.xml";
const char* cascade_name_eye_small = "/usr/local/share/OpenCV/haarcascades/haarcascade_mcs_eyepair_small.xml";
const char* cascade_name_eye_big ="/usr/local/share/OpenCV/haarcascades/haarcascade_mcs_eyepair_big.xml";

class Region_of_interest {
public:
    Region_of_interest();
    Region_of_interest(string image , bool isface , int face_recheck, int eyes );
    void setface(bool face){ face_p = face; }
    void setname(string image){ image_name = image; }
    void setface_recheck(int face_recheck){ number_face_recheck = face_recheck; }
    void seteyes(int eyes){ number_eyes = eyes;}
    string getname(){ return image_name; }
    int getIsface(){ return face_p; }
    int getface_recheck(){ return number_face_recheck; }
    int geteyes(){ return number_eyes; }
    
    void setconfirmation_face(bool face) { confirmation_face = face; }
    void seteyepair_large( bool eyes ){ eyes_large_classifier = eyes; }
    void seteyepair_small( bool eyes ){ eyes_small_classifier = eyes; }
    
    bool getconfirmation_face() { return confirmation_face; }
    bool geteyepair_large() { return eyes_large_classifier; }
    bool geteyepair_small() { return eyes_small_classifier; }
    
    
private:
    string image_name;
    bool face_p;
    bool confirmation_face;
    bool eyes_large_classifier;
    bool eyes_small_classifier;
    int number_face_recheck;
    int number_eyes;
};

Region_of_interest::Region_of_interest()
{
    image_name = "default";
    face_p = false;
    number_face_recheck =0;
    number_eyes = 0;
    confirmation_face = false;
    eyes_large_classifier = false;
    eyes_small_classifier = false;
}

Region_of_interest::Region_of_interest(string image , bool isface , int face_recheck, int eyes)
{
    image_name = image;
    face_p = isface;
    number_face_recheck = face_recheck;
    number_eyes = eyes;
}

void detect_and_draw_face( IplImage* image, const char* cascade_type, bool Show_ROI,
                          CvScalar color, vector<Region_of_interest> & face_vector);
int detect_and_draw_aspect( IplImage* image, const char* cascade_type, bool Show_ROI,
                           CvScalar color, Region_of_interest & face_OI);
void PrintInfo(vector<Region_of_interest> & face_vector);

CvScalar red = CV_RGB( 255, 0, 0);
CvScalar blue = CV_RGB( 0, 255, 0);
CvScalar green = CV_RGB( 0, 0, 255);
CvScalar white = CV_RGB( 255, 255, 255);

int main (){
    
    vector<Region_of_interest> face_vector;
    
    string imagefile = "DataPictures/ImageDirectory.txt";
    string datastring;
    IplImage *img;
    
    ifstream directoryfile;                 //image file directory
    directoryfile.open(imagefile.c_str());
    
    if (!directoryfile.is_open())
    {
        cout << "-failed to open directory file-\n";
        exit(1);
    }
    while (directoryfile.is_open()) {
        while (directoryfile.good())
        {
            directoryfile >> datastring;
            
            cout << " For image: ";
            cout << datastring << endl;
            cout << "------------------------" << endl;
            img = cvLoadImage(datastring.c_str());
            
            if(!img )                              // Check for invalid input
            {
                cout <<  "Could not open or find the image" << endl ;
                exit(1);
            }

	    detect_and_draw_face( img , cascade_name,Show_ROI_bool, red , face_vector );

            cvReleaseImage(&img);
            cout << "------------------------" << endl;
        }
        directoryfile.close();
    }

    PrintInfo(face_vector);
    cvReleaseImage(&img);
    cvDestroyWindow("result");
    
    // return 0 to indicate successfull execution of the program
    return 0;
}

void PrintInfo(vector<Region_of_interest> & face_vector)
{
    double confirmed_face = 0;
    double not_confirmed_face = 0;
    double number_eyes_in_confirmed = 0;
    double number_face_recheck_in_confirmed = 0;
    double number_eyes_in_Not_confirmed = 0;
    double number_face_recheck_in_Not_confirmed = 0;
    double avg_eyes_confirmed =0;
    double avg_face_recheck_confirmed =0;
    double avg_eyes_not_confirmed =0;
    double avg_face_recheck_not_confirmed =0;
    for ( unsigned int i= 0 ; i != face_vector.size(); i++  )
    {
        if ( face_vector[i].getIsface())
        {
            confirmed_face++;
            number_eyes_in_confirmed = number_eyes_in_confirmed+face_vector[i].geteyes();
            number_face_recheck_in_confirmed =number_face_recheck_in_confirmed+face_vector[i].getface_recheck();
        }
        else
        {
            number_eyes_in_Not_confirmed =number_eyes_in_Not_confirmed+face_vector[i].geteyes();
            number_face_recheck_in_Not_confirmed =number_face_recheck_in_Not_confirmed+face_vector[i].getface_recheck();
        }
        
    }
    cout << "-------------------------------" << endl;
    cout << "Report:\n number of Possible faces: " << face_vector.size() << "\n";
    cout << " Faces confirmed: " << confirmed_face << endl;
    
    avg_eyes_confirmed = number_eyes_in_confirmed/confirmed_face;
    avg_face_recheck_confirmed = number_face_recheck_in_confirmed/confirmed_face;
    cout << " Total number of eye pairs in confirmed: " << number_eyes_in_confirmed << endl;
    cout << " Total number of face_recheck in confirmed: " << number_face_recheck_in_confirmed << endl;
    
    cout << " Avg number of eye pairs in confirmed: " << avg_eyes_confirmed << endl;
    cout << " Avg number of face_recheck in confirmed: " << avg_face_recheck_confirmed << endl;
    
    not_confirmed_face = face_vector.size() - confirmed_face;
    avg_eyes_not_confirmed = number_eyes_in_Not_confirmed/not_confirmed_face;
    avg_face_recheck_not_confirmed = number_face_recheck_in_Not_confirmed/not_confirmed_face;
    
    cout << " Total number of eye pairs in non-faces: " << number_eyes_in_Not_confirmed << endl;
    cout << " Total number of face_recheck in non-faces: " << number_face_recheck_in_Not_confirmed << endl;
    
    cout << " Avg number of eye pairs in non-faces: " << avg_eyes_not_confirmed << endl;
    cout << " Avg number of face_recheck in non-faces: " << avg_face_recheck_not_confirmed << endl;
}

int detect_and_draw_feature( IplImage* ROI, const char* cascade_type, bool Show_ROI,
                            CvScalar color , Region_of_interest & face_OI)
{   
    static CvMemStorage* storage = 0;
    static CvHaarClassifierCascade* cascade_feature = 0;
    int scale = 1;
    
    IplImage* temp1 = cvCreateImage( cvSize(300,300),
                                    ROI->depth, ROI->nChannels);
    cvResize(ROI,temp1);
    
    cvSmooth(temp1,temp1,CV_GAUSSIAN, 9, 0, 0, 0);
    
    CvPoint pt_1, pt_2;
    cascade_feature = (CvHaarClassifierCascade*)cvLoad( cascade_type, 0, 0, 0 );
    
    if (!cascade_feature)
    {
        cout << "-Failure- error loading classifier\n";
        cout << "is directory correct: " << cascade_type << endl;
        return -1000;
    }
        
    // Allocate the memory storage
    storage = cvCreateMemStorage(0);
    cvClearMemStorage( storage );
        
    if( cascade_feature )
    {
        CvSeq* feature = cvHaarDetectObjects( temp1, cascade_feature, storage,
                                            1.1, 2, CV_HAAR_DO_CANNY_PRUNING,
                                            cvSize(40, 40) );
    
            for(int i = 0; i < (feature ? feature->total : 0); i++ )
            {
                if ( Show_ROI == true)
                {
                CvRect* r = (CvRect*)cvGetSeqElem( feature, i );
                
                pt_1.x = r->x*scale;
                pt_2.x = (r->x+r->width)*scale;
                pt_1.y = r->y*scale;
                pt_2.y = (r->y+r->height)*scale;
                
                cvRectangle( temp1, pt_1, pt_2, color, 3, 8, 0 );
                }
  
                if (cascade_type == cascade_name_eye_big)
                {
                    face_OI.seteyes(i+1);
                    face_OI.seteyepair_large(1);
                    cout << "   Large eye pair: found" << endl;
                    
                }
                
                if (cascade_type == cascade_name_eye_small)
                {
                    face_OI.seteyes(i+1);
                    face_OI.seteyepair_small(1);
                    cout << "   Small eye pair: found" << endl;
                }
                
                if (cascade_type == cascade_name)
                {
                    face_OI.setconfirmation_face(1);
                    face_OI.setface_recheck(i+1);
                    cout << "   Face confirmation: found" << endl;
                }
                
            }
        }

    if ( Show_ROI == true)
    {
        cvShowImage( cascade_type, temp1 );
        cvReleaseImage( &temp1 );
        cvDestroyWindow(cascade_type);
    }
    cvReleaseImage( &temp1 );
    cvDestroyWindow(cascade_type);
    return 1;

}

void detect_and_draw_face( IplImage* img, const char* cascade_type, bool Show_ROI,
                          CvScalar color, vector<Region_of_interest> & face_vector )
{
    {
        if ( Show_ROI == true)
        {
            cvShowImage( "region of Interest", img);
        }
        static CvMemStorage* storage = 0;
        static CvHaarClassifierCascade* cascade_feature = 0;
        int scale = 1;
        

        CvPoint pt_1, pt_2;
        CvPoint ptr1,ptr2;
        
        //load Haar Classifier
        cascade_feature = (CvHaarClassifierCascade*)cvLoad( cascade_type, 0, 0, 0 );
        
        if (!cascade_feature)
        {
            cout << "-Failure- error loading classifier\n";
            cout << "is directory correct: " << cascade_type << endl;
            }
            
            storage = cvCreateMemStorage(0);
            cvClearMemStorage( storage );
            
            if( cascade_feature )
            {
                CvSeq* feature = cvHaarDetectObjects( img, cascade_feature, storage,
                                                     1.1, 2, CV_HAAR_DO_CANNY_PRUNING,
                                                     cvSize(40, 40) );
                
                for( int i = 0; i < (feature ? feature->total : 0); i++ )
                {

                    CvRect* r = (CvRect*)cvGetSeqElem( feature, i );
                    
                    pt_1.x = r->x*scale;
                    pt_2.x = (r->x+r->width)*scale;
                    pt_1.y = r->y*scale;
                    pt_2.y = (r->y+r->height)*scale;
                    
                    ptr1.x = r->x;
                    ptr2.x = (r->x+r->width);
                    ptr1.y = r->y;
                    ptr2.y = (r->y+r->height);
                    
                    cvSetImageROI(img, cvRect(ptr1.x, ptr1.y,r->width, r->height));
                    
                    Region_of_interest * New_face = new Region_of_interest();
                    New_face->setname("image1");
                    
                    cout << "   Possiable face found: " << ptr1.x <<" " << ptr1.y;
                    cout <<" " << ptr2.x  <<" " <<ptr2.y << endl;
                    cout << "***********************" << endl;
        
                    detect_and_draw_feature( img , cascade_name,Show_ROI_bool, green ,*New_face);
                    detect_and_draw_feature(img,cascade_name_eye_small,Show_ROI_bool , blue, *New_face);
                    detect_and_draw_feature(img,cascade_name_eye_big, Show_ROI_bool, white, *New_face);
                    
                    cvResetImageROI(img);
          
                    if (New_face->getconfirmation_face() && (New_face->geteyepair_large()
                                                             ||New_face->geteyepair_small()))
                    {
                        New_face->setface(1);
                        cvRectangle( img, pt_1, pt_2, color, 1.8, 8, 0 );
                    }
         
                    
                    face_vector.push_back(*New_face);
                    delete New_face;
                }
            }
    if ( Show_ROI == true)
	{
	cvShowImage( "result", img );
        
    }
    }
}
